﻿(function () {
    'use strict';

    var MaterialSelectfield = function MaterialSelectfield(element) {
        this.element_ = element;
        // Initialize instance.
        this.init();
    };
    window['MaterialSelectfield'] = MaterialSelectfield;

    MaterialSelectfield.prototype.Constant_ = {
        // None for now
    };

    MaterialSelectfield.prototype.CssClasses_ = {
        LABEL: 'mdl-selectield__label',
        SELECT: 'mdl-selectfield__select',
        IS_DIRTY: 'is-dirty',
        IS_FOCUSED: 'is-focused',
        IS_DISABLED: 'is-disabled',
        IS_INVALID: 'is-invalid',
        IS_UPGRADED: 'is-upgraded'
    };

    /**
     * Handle focus.
     *
     * @param {Event} event The event that fired.
     * @private
     */
    MaterialSelectfield.prototype.onFocus_ = function (event) {
        this.element_.classList.add(this.CssClasses_.IS_FOCUSED);
    };

    /**
     * Handle lost focus.
     *
     * @param {Event} event The event that fired.
     * @private
     */
    MaterialSelectfield.prototype.onBlur_ = function (event) {
        this.element_.classList.remove(this.CssClasses_.IS_FOCUSED);
    };

    /**
     * Handle reset event from outside.
     *
     * @param {Event} event The event that fired.
     * @private
     */
    MaterialSelectfield.prototype.onReset_ = function (event) {
        this.updateClasses_();
    };

    /**
     * Handle class updates.
     *
     * @private
     */
    MaterialSelectfield.prototype.updateClasses_ = function () {
        this.checkDisabled();
        this.checkValidity();
        this.checkDirty();
    };

    // Public methods.

    /**
     * Check the disabled state and update field accordingly.
     *
     * @public
     */
    MaterialSelectfield.prototype.checkDisabled = function () {
        if (this.select_.disabled) {
            this.element_.classList.add(this.CssClasses_.IS_DISABLED);
        } else {
            this.element_.classList.remove(this.CssClasses_.IS_DISABLED);
        }
    };
    MaterialSelectfield.prototype['checkDisabled'] = MaterialSelectfield.prototype.checkDisabled;

    /**
     * Check the validity state and update field accordingly.
     *
     * @public
     */
    MaterialSelectfield.prototype.checkValidity = function () {
        if (this.select_.validity.valid) {
            this.element_.classList.remove(this.CssClasses_.IS_INVALID);
        } else {
            this.element_.classList.add(this.CssClasses_.IS_INVALID);
        }
    };
    MaterialSelectfield.prototype['checkValidity'] = MaterialSelectfield.prototype.checkValidity;

    /**
     * Check the dirty state and update field accordingly.
     *
     * @public
     */
    MaterialSelectfield.prototype.checkDirty = function () {
        if (this.select_.value && this.select_.value.length > 0) {
            this.element_.classList.add(this.CssClasses_.IS_DIRTY);
        } else {
            this.element_.classList.remove(this.CssClasses_.IS_DIRTY);
        }
    };
    MaterialSelectfield.prototype['checkDirty'] = MaterialSelectfield.prototype.checkDirty;

    /**
     * Enable select field.
     *
     * @public
     */
    MaterialSelectfield.prototype.disable = function () {
        this.select_.disabled = true;
        this.updateClasses_();
    };
    MaterialSelectfield.prototype['disable'] = MaterialSelectfield.prototype.disable;

    /**
     * Enable select field.
     *
     * @public
     */
    MaterialSelectfield.prototype.enable = function () {
        this.select_.disabled = false;
        this.updateClasses_();
    };
    MaterialSelectfield.prototype['enable'] = MaterialSelectfield.prototype.enable;

    /**
     * Update select field value.
     *
     * @param {string} value The value to which to set the control (optional).
     * @public
     */
    MaterialSelectfield.prototype.change = function (value) {
        if (value) {
            this.select_.value = value;
        }
        this.updateClasses_();
    };
    MaterialSelectfield.prototype['change'] = MaterialSelectfield.prototype.change;

    /**
     * Initialize element.
     */
    MaterialSelectfield.prototype.init = function () {
        if (this.element_) {
            this.label_ = this.element_.querySelector('.' + this.CssClasses_.LABEL);
            this.select_ = this.element_.querySelector('.' + this.CssClasses_.SELECT);

            if (this.select_) {
                this.boundUpdateClassesHandler = this.updateClasses_.bind(this);
                this.boundFocusHandler = this.onFocus_.bind(this);
                this.boundBlurHandler = this.onBlur_.bind(this);
                this.boundResetHandler = this.onReset_.bind(this);
                this.select_.addEventListener('change', this.boundUpdateClassesHandler);
                this.select_.addEventListener('focus', this.boundFocusHandler);
                this.select_.addEventListener('blur', this.boundBlurHandler);
                this.select_.addEventListener('reset', this.boundResetHandler);

                this.updateClasses_();
                this.element_.classList.add(this.CssClasses_.IS_UPGRADED);
            }
        }
    };

    /**
     * Downgrade the component
     *
     * @private
     */
    MaterialSelectfield.prototype.mdlDowngrade_ = function () {
        this.select_.removeEventListener('change', this.boundUpdateClassesHandler);
        this.select_.removeEventListener('focus', this.boundFocusHandler);
        this.select_.removeEventListener('blur', this.boundBlurHandler);
        this.select_.removeEventListener('reset', this.boundResetHandler);
    };

    // The component registers itself. It can assume componentHandler is available
    // in the global scope.
    componentHandler.register({
        constructor: MaterialSelectfield,
        classAsString: 'MaterialSelectfield',
        cssClass: 'mdl-js-selectfield',
        widget: true
    });

})();
