angular.module('app.services', [])

.factory('GuardiaFactory', ['$http', '$q', '$cordovaFileTransfer', '$window', '$timeout', 'ApiEndpoint', 'APPVERSION', function ($http, $q, $cordovaFileTransfer, $window, $timeout, ApiEndpoint, APPVERSION) {
    var transaction = {};
    transaction.URL = ApiEndpoint.url;
    transaction.app_version = APPVERSION.version;
    function String2Hex(tmp){
        var str = '';
        for(var i = 0; i < tmp.length; i++) {
            str += tmp[i].charCodeAt(0).toString(16);
        }
        return str;
    }
    function _arrayBufferToBase64(buffer) {
        var binary = '';
        var bytes = new Uint8Array(buffer);
        var len = bytes.byteLength;
        for (var i = 0; i < len; i++) {
          binary += String.fromCharCode(bytes[i]);
        }
        return window.btoa(binary);
      }
    function deleteFileImage(path, callback, errorCall){
        window.resolveLocalFileSystemURL(path, gotFile, fail);

        function fail(e) {
            errorCall(3);
        }

        function gotFile(fileEntry) {
            fileEntry.remove(function (file) {
                console.log("file removed!");
                callback();
            }, function (error) {
                console.log("error occurred: " + error.code);
            }, function () {
                console.log("file does not exist");
            });
        }
    }
    function getFileImageAsBase64(path, callback, errorCall) {
        window.resolveLocalFileSystemURL(path, gotFile, fail);

        function fail(e) {
            errorCall(3);
        }

        function gotFile(fileEntry) {
            fileEntry.file(function (file) {
                var reader = new FileReader();
                reader.onloadend = function (e) {
                    var content = this.result;
                    callback(content);
                };
                // The most important point, use the readAsDatURL Method from the file plugin
                reader.readAsDataURL(file);
            });
        }
    }
    function getRandomString(length) {
        var randomChars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
        var result = '';
        for ( var i = 0; i < length; i++ ) {
            result += randomChars.charAt(Math.floor(Math.random() * randomChars.length));
        }
        return result;
    }

    transaction.login = function (Correo, Password, IMEI) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v2/usuario/login', {
                email: Correo,
                password: Password,
                imei: IMEI
            }
        ).success(function (data) {            
            deferredObject.resolve({ success: true, datos: data });            
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.checkVersion = function () {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v1/caseta/version', {
                package: 'com.kolonus.guardia',
                plataforma: 'android',
                versionNumber: transaction.app_version
            }
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    }

    transaction.cambioPassword = function (Correo, OldPassword, NewPassword) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v1/usuario/password', {
                email: Correo,
                oldPassword: OldPassword,
                newPassword: NewPassword
            }
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.pushConfig = function (_uuid_cordova, _uuid_onesignal, _pushToken_onesignal) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v1/usuario/pushConfig', {
                uuid_cordova: _uuid_cordova,
                uuid_onesignal: _uuid_onesignal,
                pushToken_onesignal: _pushToken_onesignal
            }
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.recuperarPassword = function (correo) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v1/usuario/recuperar', {
                email: correo
            }
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    }

    transaction.buscarVisitas = function (IdSector, EsEntrada) {
        var deferredObject = $q.defer();
        $http.get(
            transaction.URL + 'api/v1/caseta/buscar/visita/' + IdSector + '/' + EsEntrada
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.buscarEntradasAutorizadas = function (IdSector, filtro) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v2/caseta/buscar/entradas/' + IdSector,
            {
                searchTerm: filtro
            }
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.buscarSalidasPendientes = function (IdSector, filtro) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v2/caseta/buscar/salidas/' + IdSector,
            {
                searchTerm: filtro
            }
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.buscarCodigo = function (IdSector, Codigo, EsEntrada) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v2/caseta/buscar/codigo', {
                id_sector: IdSector,
                codigo: Codigo,
                esEntrada: EsEntrada
            }
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.buscarNombre = function (IdSector, Texto, EsEntrada) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v1/caseta/buscar/nombre', {
                id_sector: IdSector,
                texto: Texto,
                esEntrada: EsEntrada
            }
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.registrarEntrada = function (IdBitacora, IdVisitante, IdTipoRegistro, IdTipoId, Identificacion, Placas, Marca, Modelo, Color, IdUsuario, CantidadPersonas, idAsunto) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v3/caseta/in', {
                id_bitacora: IdBitacora,
                id_visitante: IdVisitante,
                id_tipoRegistro: IdTipoRegistro,
                id_tipoId: IdTipoId,
                identificacion: Identificacion,
                placas: Placas,
                marca: Marca,
                modelo: Modelo,
                color: Color,
                id_usuario: IdUsuario,
                cantidadPersonas: CantidadPersonas,
                id_asunto: idAsunto
            }
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.registrarEntradaV7 = function (IdBitacora, IdVisitante, IdTipoRegistro, IdTipoId, Identificacion, Placas, Generales, Tag, IdUsuario, CantidadPersonas, idAsunto, imagen1, imagen2) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v7/caseta/in', {
                id_bitacora: IdBitacora,
                id_visitante: IdVisitante,
                id_tipoRegistro: IdTipoRegistro,
                id_tipoId: IdTipoId,
                identificacion: Identificacion,
                placas: Placas,
                generales: Generales,
                tag: Tag,
                id_usuario: IdUsuario,
                cantidadPersonas: CantidadPersonas,
                id_asunto: idAsunto,
                foto1: imagen1,
                foto2: imagen2
            }
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.registrarSalida = function (IdVisitante, IdUsuario, VaRegresar, IdTipoRegistro) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v1/caseta/out', {
                id_visitante: IdVisitante,
                id_usuario: IdUsuario,
                vaRegresar: VaRegresar,
                id_tipoRegistro: IdTipoRegistro
            }
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.visitanteImprevisto = function (Nombres, Apellidos, IdTipoId, Identificacion, Placa, IdUsuario, IdCasa, IdTipoVisitante, IdSector, Marca, Modelo, Color, CantidadPersonas, IdAsunto) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v3/caseta/visitante', {
                nombres: Nombres,
                apellidos: Apellidos,
                id_tipo_id: IdTipoId,
                identificacion: Identificacion,
                placa: Placa,
                id_usuario: IdUsuario,
                id_casa: IdCasa,
                id_tipo_visitante: IdTipoVisitante,
                id_sector: IdSector,
                marca: Marca,
                modelo: Modelo,
                color: Color,
                cantidadPersonas: CantidadPersonas,
                id_asunto: IdAsunto
            }
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };
    
    transaction.visitanteImprevistoV7 = function (Nombres, Apellidos, IdTipoId, Identificacion, Placa, IdUsuario, IdCasa, IdTipoVisitante, IdSector, Generales, Tag, CantidadPersonas, IdAsunto, imagen1, imagen2) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v7/caseta/visitante', {
                nombres: Nombres,
                apellidos: Apellidos,
                id_tipo_id: IdTipoId,
                identificacion: Identificacion,
                placa: Placa,
                id_usuario: IdUsuario,
                id_casa: IdCasa,
                id_tipo_visitante: IdTipoVisitante,
                id_sector: IdSector,
                generales: Generales,
                tag: Tag,                
                cantidadPersonas: CantidadPersonas,
                id_asunto: IdAsunto,
                foto1: imagen1,
                foto2: imagen2
            }
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };
    
    transaction.listaCasas = function (IdSector) {
        var deferredObject = $q.defer();
        $http.get(
            transaction.URL + 'api/v3/caseta/consulta/casa/' + IdSector,
            { cache: false}
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.listaMorosidad = function (IdSector) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v1/caseta/consulta/morosidad', {
                id_sector: IdSector
            }
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.listaMtto = function (IdSector) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v1/caseta/consulta/mantenimiento', {
                id_sector: IdSector
            }
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.listaIncidente = function (IdSector, IdCuenta) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v2/consulta/incidente', {
                id_sector: IdSector,
                id_cuenta: IdCuenta
            }
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.listaComunicado = function (Idcuenta) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v1/caseta/consulta/comunicado', {
                id_cuenta: Idcuenta
            }
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.leerComunicado = function (idDetComunicado) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v1/caseta/comunicado/leido/' + idDetComunicado
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.catalogoTipoId = function () {
        var deferredObject = $q.defer();
        $http.get(
            transaction.URL + 'api/v1/caseta/catalogo/tipoId',
            { cache: true}
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.obtenerBadges = function (idSector, idCuenta) {
        var deferredObject = $q.defer();
        var endpointUrl = transaction.URL + 'api/v1/caseta/consulta/badges/' + idSector + '/' + idCuenta;
        $http.get(
            endpointUrl
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.listaAsuntos = function (IdSector) {
        var deferredObject = $q.defer();
        $http.get(
            transaction.URL + 'api/v3/caseta/catalogo/asunto',
            { cache: true}
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };
    
    transaction.getTwilioToken = function (page, endpoint) {
        var twUrl = "https://violet-budgerigar-1188.twil.io/access-token";
        if(endpoint)
            twUrl = endpoint;

        var deferredObject = $q.defer();
        $http.post(
            twUrl,
            { page: page},
            {headers: {'Content-Type': 'application/x-www-form-urlencoded'}}
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };
    
    /*
   transaction.getTwilioToken = function (page) {
        var deferredObject = $q.defer();
        $http.post(
            'https://violet-budgerigar-1188.twil.io/token-test',
            { page: page},
            {headers: {'Content-Type': 'application/x-www-form-urlencoded'}}
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };
    */
    transaction.saveTwilioCall = function (pSid, idSector, IdCasa) {
        var deferredObject = $q.defer();
        $http.post(
            transaction.URL + 'api/v3/caseta/llamada', {
                Sid: pSid,
                id_sector: idSector,
                id_casa: IdCasa
            }
        ).success(function (data) {
            deferredObject.resolve({ success: true, datos: data });
        }).error(function (response) {
            deferredObject.resolve({ success: false, datos: response })
        });
        return deferredObject.promise;
    };

    transaction.subirFotoEntrada = function (idSector, idCuenta, IdCasa, idTabla, idTipo, imageUri, token) {
        console.log("subirFotoEntrada");
        var deferredObject = $q.defer();
        var options = {
            fileKey: "evidencia",
            fileName: "image.jpg",
            chunkedMode: false,
            mimeType: "image/jpeg",
            headers: {
                'Connection': 'close',
                'Authorization': 'Bearer ' + token                
            }
        };
        if (IdCasa == null)
            IdCasa = 0;
        if (IdCasa == "")
            IdCasa = 0;
        var server = transaction.URL + 'api/v2/caseta/in/archivo/' + idSector + '/' + idCuenta + '/' + IdCasa + '/' + idTabla + '/' + idTipo
        $cordovaFileTransfer.upload(server, imageUri, options)
          .then(function (result) {
              console.log("subio archivo");
              deferredObject.resolve({ success: true, datos: result.response });
          }, function (err) {
              console.log("Fallo archivo");
              deferredObject.resolve({ success: false, datos: err })
          }, function (progress) {
              // constant progress updates
          });
        return deferredObject.promise;
    }

    transaction.saveStatusCamara = function (pantalla, datos) {
        datos.pantalla = pantalla;
        datos.imageURI = null;
        datos.imageURI2 = null;
        return $q(function(resolve, reject) {            
            var json = JSON.stringify(datos);
            $window.plugins.appPreferences.store(function (value) {
                console.log("Valor Almacenado");
                return resolve();
            }, function (error) {
                console.log("Error " + error);
                return reject();
            }, 'camara', 'statusCamara', json);
          });
    };
    
    transaction.saveStatusCamaraEntrada = function (pantalla, datos) {
        datos.registroForm.imageURI2 = '';
        datos.registroForm.imageURI = '';
        datos.pantalla = pantalla;        
        return $q(function(resolve, reject) {            
            var json = JSON.stringify(datos);
            $window.plugins.appPreferences.store(function (value) {
                console.log("Valor Almacenado");
                return resolve();
            }, function (error) {
                console.log("Error " + error);
                return reject();
            }, 'camara', 'statusCamara', json);
          });
    };

    transaction.cleanStatusCamara = function () {
        
        return $q(function(resolve, reject) {            
            $window.plugins.appPreferences.remove(function (value) { return resolve(); }, function (error) { return reject(); }, 'camara', 'statusCamara');
          });
    };
    
    transaction.getStatusCamara = function (){
        return $q(function(resolve, reject) {   
        $window.plugins.appPreferences.fetch(
            function (value) {
                if(value != null)
                {
                    console.log(value);
                    var obj = JSON.parse(value);
                    console.log(obj);
                    return resolve(obj);
                }else{
                    return reject();
                }
            }, function (error) {

                return reject();
            }, 'camara', 'statusCamara');
        });
    }
    
    transaction.getSesion = function (key, storage){
        return $q(function(resolve, reject) {   
        $window.plugins.appPreferences.fetch(
            function (value) {
                return resolve(value);
            }, function (error) {
                return reject();
            }, storage, key);
        });
    }

    transaction.getProvisionSnapshot = function (_camaraIp, user, pass) {
        var deferredObject = $q.defer();
        var filename = getRandomString(24) + '.jpeg';
        var token = btoa(String(user) + ':' + String(pass));
        var targetPath = cordova.file.dataDirectory + filename;
        var camaraUrl= 'http://' + String(_camaraIp) + '/GetSnapshot/1';
        console.log(camaraUrl);
        var options = {
            headers: {
                'Authorization': 'Basic ' + token,
                'Content-Type': 'application/octet-stream'
            },
            timeout: 15000
        }
        var trustHosts = true       
        
        $cordovaFileTransfer.download(camaraUrl, targetPath, options, trustHosts)
            .then(function(result) {               
                getFileImageAsBase64(result.nativeURL, function (base64Image) {                      
                    deleteFileImage(result.nativeURL, function(){console.log("eliminando")}, function(err){});
                    deferredObject.resolve({ success: true, datos: base64Image });               
                }, function (err){
                    deferredObject.resolve({ success: false, datos: err })
                });
            }, function(error) {                     
                deferredObject.resolve({ success: false, datos: JSON.stringify(error) })
            });            
        return deferredObject.promise;        
    };

    transaction.getHikvisionSnapshot = function (_camaraIp, user, pass) {
        var deferredObject = $q.defer();
        var filename = getRandomString(24) + '.jpeg';
        var token = btoa(String(user) + ':' + String(pass));
        var targetPath = cordova.file.dataDirectory + filename;
        var camaraUrl= 'http://' + String(_camaraIp) + '/ISAPI/Streaming/channels/1/picture';
        console.log(camaraUrl);
        var options = {
            headers: {
                'Authorization': 'Basic ' + token,
                'Content-Type': 'application/octet-stream'
            },
            timeout: 15000
        }
        var trustHosts = true       
        
        $cordovaFileTransfer.download(camaraUrl, targetPath, options, trustHosts)
            .then(function(result) {               
                getFileImageAsBase64(result.nativeURL, function (base64Image) {                      
                    deleteFileImage(result.nativeURL, function(){console.log("eliminando")}, function(err){});
                    deferredObject.resolve({ success: true, datos: base64Image });               
                }, function (err){
                    deferredObject.resolve({ success: false, datos: err })
                });
            }, function(error) {                     
                deferredObject.resolve({ success: false, datos: JSON.stringify(error) })
            });            
        return deferredObject.promise;                
        
    };
    
    
    return transaction;

    
}])

.service('BlankService', [function(){

}]);

